/**
 * Created by lanhao on 16/6/7.
 */

'use strict';

const request = require('request');
const Entity = require('./libs/requestEntity');

let index = {};

index.reset = index.init = () => {
  this.entity = new Entity();
  return index;
};

index.toObject = index.toSource = index.toString = () => this.entity;

index.headers = (headers, flush = false) => {
  this.entity.headers = this.entity.headers || {};
  if (!!flush) {
    this.entity.headers = {};
  }
  for (let k in headers) {
    this.entity.headers[k] = headers[k];
  }
  return index;
};

index.query = (query, flush = false) => {
  this.entity.query = this.entity.query || {};
  if (!!flush) {
    this.entity.query = {};
  }
  for (let k in query) {
    this.entity.query[k] = query[k];
  }
  return index;
};

index.body = (body, flush = false) => {
  this.entity.body = this.entity.body || {};
  if (!!flush) {
    this.entity.body = {};
  }
  for (let k in body) {
    this.entity.body[k] = body[k];
  }
  return index;
};

index.url = (url) => {
  this.entity.url = url;
  return index;
};

index.method = (method) => {
  method = (method + '').toLowerCase();
  if (this.entity.hasMethod(method)) {
    this.entity.method = method;
  } else {
    console.error('not support method : ' + method);
  }
  return index;
};

index.send = (reset = true) => {
  let _o = this.entity.format();
  let promise = new Promise((resolve, reject) => {
    request(_o, (e, r) => {
      !!reset && index.reset();
      if (!e) {
        resolve(r);
      } else {
        reject(e);
      }
    });
  });
  return promise;
};

index.toJson = (res) => {
  let body = res.body;
  try {
    body = JSON.parse(body);
  } catch (ex) {

  }
  return body;
};

index.isOk = (res) => {
  let statusCode = res.statusCode;
  return statusCode === 200 ? true : false;
};

index.toCURL = () => {
  let _o = this.entity.format();
  var url = _o.url;
  if (!url) {
    return false;
  }

  let command = 'curl ' + url + ' ';

  if (_o.method) {
    command += ' -X' + _o.method.toUpperCase() + ' ';
  }

  if (_o.headers) {
    for (let k in _o.headers) {
      command += '-H "' + k.toLowerCase() + ':' + _o.headers[k] + '" ';
    }
  }

  if (_o.form || _o.json) {
    command += '-d \'' + (_o.form ? _o.form : JSON.stringify(_o.json)) + '\' ';
  }
  return command.replace('&', '\\&');
};


module.exports = index;
